﻿using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.Hosting.WindowsServices;
using Parsing_Packets.Models;

namespace Parsing_Packets;

public class Program
{

    internal static async Task Main(string[] args)
    {
        ILogger<Program>? logger = null;
#pragma warning disable IL3050
        WebApplicationBuilder webApplicationBuilder = WebApplication.CreateBuilder(args);
#pragma warning restore IL3050
        _ = webApplicationBuilder.Configuration.AddUserSecrets<Program>();
        PhysicalAddressConfiguration physicalAddressConfiguration = Models.Binder.PhysicalAddressConfiguration.Get(webApplicationBuilder.Configuration);
        AppSettings appSettings = Models.Binder.AppSettings.Get(webApplicationBuilder.Configuration,
                                                                physicalAddressConfiguration);
        if (string.IsNullOrEmpty(appSettings.Company))
            throw new Exception("Company name must have a value!");
        try
        {
            List<string> collection = [];
            _ = webApplicationBuilder.Services.AddHostedService<Worker>();
            _ = webApplicationBuilder.Services.AddSingleton(collection);
            _ = webApplicationBuilder.Services.AddSingleton(appSettings);
            if (WindowsServiceHelpers.IsWindowsService())
            {
                collection.Add(nameof(WindowsServiceLifetime));
                _ = webApplicationBuilder.Services.AddSingleton<IHostLifetime, WindowsServiceLifetime>();
                _ = webApplicationBuilder.Logging.AddEventLog(settings =>
                {
#pragma warning disable CA1416
                    if (string.IsNullOrEmpty(settings.SourceName))
                        settings.SourceName = webApplicationBuilder.Environment.ApplicationName;
#pragma warning restore
                });
            }
            using WebApplication webApplication = webApplicationBuilder.Build();
            logger = webApplication.Services.GetRequiredService<ILogger<Program>>();
            if (string.IsNullOrEmpty(appSettings.Company))
            {
                Environment.ExitCode = -1;
                _ = webApplication.StopAsync();
            }
            logger.LogInformation("Starting Web Application");
            logger.LogCritical("{Company}", appSettings.Company);
            await webApplication.RunAsync();
        }
        catch (Exception ex)
        {
            try
            { logger?.LogCritical(ex, "WebApplication terminated unexpectedly"); }
            catch (Exception) { }
            throw;
        }
    }

}